@@ -17,20 +17,43 @@
 --]]
 
 -- Version information: should be identical to that in l3build.dtx
-release_date = "2014/06/15"
-release_ver  = "5141"
+release_date = "2014/09/15"
+release_ver  = "5423"
+
+-- "module" is a deprecated function in Lua 5.2: as we want the name
+-- for other purposes, and it should eventually be 'free', simply
+-- remove the built-in
+if type (module) == "function" then
+  module = nil
+end
 
 -- Ensure the module and bundle exist
 module = module or ""
 bundle = bundle or ""
 
+-- Sanity check
+if module == "" and bundle == "" then
+  if string.match (arg[0], ".*l3build%.lua$") then
+    print (
+        "\n" ..
+        "Error: Call l3build using a configuration file, not directly.\n"
+      )
+  else
+    print (
+      "\n" ..
+      "Error: Specify either bundle or module in configuration script.\n"
+      )
+  end
+  os.exit (1)
+end
+
 -- Directory structure for the build system
 -- Use Unix-style path separators
 maindir     = maindir or "."
 
 -- Substructure for tests and support files
 testfiledir = testfiledir or "testfiles" -- Set to "" to cancel any tests
-testsupdir  = testsupdir  or testfiledir .. "/support"
+testsuppdir = testsuppdir or testfiledir .. "/support"
 supportdir  = supportdir  or maindir .. "/support"
 
 -- Structure within a development area
@@ -64,7 +87,7 @@ cleanfiles       = cleanfiles       or {"*.pdf", "*.zip"}
 excludefiles     = excludefiles     or {"*~"}             -- Any Emacs stuff
 installfiles     = installfiles     or {"*.sty"}
 sourcefiles      = sourcefiles      or {"*.dtx", "*.ins"} -- Files to copy for unpacking
-txtfiles         = txtfiles         or {"*.markdown"}
+txtfiles         = txtfiles         or {"*.md", "*.markdown"}
 typesetfiles     = typesetfiles     or {"*.dtx"}
 typesetsuppfiles = typesetsuppfiles or { }
 unpackfiles      = unpackfiles      or {"*.ins"}          -- Files to actually unpack
@@ -87,8 +110,9 @@ unpackopts  = unpackopts  or ""
 zipopts     = zipopts     or "-v -r -X"
 
 -- Engines for testing
-chkengines = chkengines or {"pdftex", "xetex", "luatex"}
-stdengine  = stdengine  or "pdftex"
+checkengines = checkengines or {"pdftex", "xetex", "luatex"}
+checkformat  = checkformat  or "latex"
+stdengine    = stdengine  or "pdftex"
 
 -- Enable access to trees outside of the repo
 -- As these may be set false, a more elaborate test than normal is needed
@@ -105,6 +129,7 @@ end
 
 -- Other required settings
 checkruns   = checkruns   or 1
+packtdszip  = packtdszip  or false -- Not actually needed but clearer
 scriptname  = scriptname  or "build.lua" -- Script used in each directory
 typesetcmds = typesetcmds or ""
 
@@ -362,9 +387,7 @@ end
 -- Set up the check system files: needed for checking one or more tests and
 -- for saving the test files
 function checkinit ()
-  cleandir (localdir)
   cleandir (testdir)
-  cleandir (unpackdir)
   depinstall (checkdeps)
   -- Copy dependencies to the test directory itself: this makes the paths
   -- a lot easier to manage, and is important for dealing with the log and
@@ -379,9 +402,9 @@ function checkinit ()
   for _,i in ipairs (checkfiles) do
     cp (i, unpackdir, testdir)
   end
-  if direxists (testsupdir) then
-    for _,i in ipairs (filelist (testsupdir)) do
-      cp (i, testsupdir, testdir)
+  if direxists (testsuppdir) then
+    for _,i in ipairs (filelist (testsuppdir)) do
+      cp (i, testsuppdir, testdir)
     end
   end
   for _,i in ipairs (checksuppfiles) do
@@ -393,8 +416,7 @@ end
 function depinstall (deps)
   for _,i in ipairs (deps) do
     print ("Installing dependency: " .. i)
-    run (i, "texlua " .. scriptname .. " localinstall")
-    cleandir (unpackdir)
+    run (i, "texlua " .. scriptname .. " unpack")
   end
 end
 
@@ -443,10 +465,18 @@ function formatlog (logfile, newfile)
     line = string.gsub (line, string.match (logfile, ".*/(.*)%" .. logext .. "$"), "")
     -- Zap ./ at begin of filename
     line = string.gsub (line, "%(%.%/", "(")
+    -- Zap paths if places other than 'here' are accessible
+    if checksearch then
+      line = string.gsub (line, "%(.*/([%w-]+%.[%w-]+)%s*$", "(../%1")
+    end
     -- XeTeX knows only the smaller set of dimension units
     line = string.gsub (
         line, "cm, mm, dd, cc, bp, or sp", "cm, mm, dd, cc, nd, nc, bp, or sp"
       )
+    -- Normalise a case where a LuaTeX bug misprints \csname\endcsname
+    -- This comes before the normalisation of the appearance of 
+    -- \csname\endcsname in case we alter that!
+    line = string.gsub (line, "\\ycsnam\\rendcsnam", "\\csname\\endcsname")
     -- Normalise a case where fixing a TeX bug changes the message text
     line = string.gsub (line, "\\csname\\endcsname ", "\\csname\\endcsname")
     -- Zap "on line <num>" and replace with "on line ..."
@@ -465,6 +495,12 @@ function formatlog (logfile, newfile)
     for i = 1, 31, 1 do
       line = string.gsub (line, string.char (i), "^^" .. string.char (64 + i))
     end
+    -- Minor LuaTeX difference: it does not include parentheses in one message
+    line = string.gsub (
+        line,
+        "%(If you're confused by all this, try typing `I}' now%.%)",
+        "If you're confused by all this, try typing `I}' now."
+      )
     -- Minor LuaTeX bug: it prints an extra "'" in one message: add enough
     -- context to hopefully hit only the bug
     line = string.gsub (
@@ -499,15 +535,34 @@ function formatlog (logfile, newfile)
   io.close (newfile)
 end
 
+-- List all modules
+function listmodules ()
+  local modules = { }
+  local exclmodules = exclmodules or { }
+  for entry in lfs.dir (".") do
+    if entry ~= "." and entry ~= ".." then
+      local attr = lfs.attributes (entry)
+      assert (type (attr) == "table")
+      if attr.mode == "directory" then
+        if not exclmodules[entry] then
+          table.insert (modules, entry)
+        end
+      end
+    end
+  end
+  return (modules)
+end
+
 -- Runs a single test: needs the name of the test rather than the .lvt file
 -- One 'test' here may apply to multiple engines
 function runcheck (name, engine, hide)
-  local chkengines = chkengines
+  local checkengines = checkengines
   if engine then
-    chkengines = {engine}
+    checkengines = {engine}
   end
   local errorlevel = 0
-  for _,i in ipairs (chkengines) do
+  for _,i in ipairs (checkengines) do
+    cp (name .. tlgext, testfiledir, testdir)
     runtest (name, i, hide)
     local testname = name .. "." .. i
     local difffile = testdir .. "/" .. testname .. os_diffext
@@ -538,7 +593,10 @@ function runtest (name, engine, hide)
   cp (name .. lvtext, testfiledir, testdir)
   local engine = engine or stdengine
   -- Engine name doesn't include the "la" for LaTeX!
-  local cmd = string.gsub (engine, "tex$", "latex")
+  local cmd = engine
+  if checkformat == "latex" then
+    cmd = string.gsub (engine, "tex$", "latex")
+  end
   local logfile = testdir .. "/" .. name .. logext
   local lvtfile = name .. lvtext
   local newfile = testdir .. "/" .. name .. "." .. engine .. logext
@@ -574,30 +632,39 @@ end
 help = help or function ()
   print ""
   if testfiledir ~= "" then
-    print " build check                    - run automated check system                "
+    print " build check                 - run all automated tests for all engines"
   end
   if module ~= "" and testfiledir ~= "" then
-    print " build checklvt <name>          - check one test file <name> for all engines"
-    print " build checklvt <name> <engine> - check one test file <name> for <engine>   "
+    print " build check <name>          - check one test file <name> for all engines"
+    print " build check <name> <engine> - check one test file <name> for <engine>   "
   end
-  print " build clean                    - clean out directory tree                  "
+  print " build clean                 - clean out directory tree                  "
   if next (cmdchkfiles) ~= nil then
-    print " build cmdcheck                 - check commands documented are defined     "
+    print " build cmdcheck              - check commands documented are defined     "
   end
   if module == "" or bundle == "" then
-    print " build ctan                     - create CTAN-ready archive                 "
+    print " build ctan                  - create CTAN-ready archive                 "
   end
-  print " build doc                      - runs all documentation files              "
-  print " build install                  - install files in local texmf tree         "
+  print " build doc                   - runs all documentation files              "
+  print " build install               - install files in local texmf tree         "
   if module ~= "" and testfiledir ~= "" then
-    print " build savetlg <name>           - save test log for <name> for all engines  "
-    print " build savetlg <name> <engine>  - save test log for <name> for <engine>     "
+    print " build save <name>           - save test log for <name> for all engines  "
+    print " build save <name> <engine>  - save test log for <name> for <engine>     "
   end
-  print " build unpack                   - extract packages                          "
   print ""
 end
 
-function check ()
+function check (name, engine)
+  local errorlevel = 0
+  if name then
+    errorlevel = checklvt (name, engine)
+  else
+    errorlevel = checkall ()
+  end
+  return (errorlevel)
+end
+
+function checkall ()
   local errorlevel = 0
   if testfiledir ~= "" and direxists (testfiledir) then
     checkinit ()
@@ -611,11 +678,7 @@ function check ()
       end
     end
     if errorlevel ~= 0 then
-      print ("\n  Check failed with difference files")
-      for _,i in ipairs (filelist (testdir, "*" .. os_diffext)) do
-        print ("  - " .. testdir .. "/" .. i)
-      end
-      print ("")
+      checkdiff ()
     else
       print ("\n  All checks passed\n")
     end
@@ -624,25 +687,38 @@ function check ()
 end
 
 function checklvt (name, engine)
-  local engine = engine or stdengine
   if testexists (name) then
     checkinit ()
     print ("Running checks on " .. name)
-    runcheck (name, engine)
-    if fileexists (testdir .. "/" .. name .. "." .. engine .. os_diffext) then
-      print ("  Check fails with diff file")
-      print (
-        "  " .. testdir .. os_dirsep .. name .. "." .. engine .. os_diffext)
+    local errorlevel = runcheck (name, engine)
+    if errorlevel ~= 0 then
+      checkdiff ()
     else
-      print ("  Check passes")
+      if engine then
+        print ("  Check passes")
+      else
+        print ("\n  All checks passed\n")
+      end
     end
   else
     print ("Test \"" .. name .. "\" not set up!")
   end
 end
 
+-- A short auxiliary to print the list of differences for check
+function checkdiff ()
+  print ("\n  Check failed with difference files")
+  for _,i in ipairs (filelist (testdir, "*" .. os_diffext)) do
+    print ("  - " .. testdir .. "/" .. i)
+  end
+  print ("")
+end
+
 -- Remove all generated files
 function clean ()
+  -- To make sure that distribdir never contains any stray subdirs,
+  -- it is entirely removed then recreated rather than simply deleting
+  -- all of the files
   rmdir (distribdir)
   mkdir (distribdir)
   cleandir (localdir)
@@ -665,7 +741,7 @@ end
 
 -- Check commands are defined
 function cmdcheck ()
-  cleandir (localdir)
+  mkdir (localdir)
   cleandir (testdir)
   depinstall (checkdeps)
   local engine = string.gsub (stdengine, "tex$", "latex")
@@ -717,15 +793,15 @@ function ctan (standalone)
   end
   local errorlevel
   if standalone then
-    clean ()
     errorlevel = check ()
     bundle = module
   else
-    bundleclean ()
     errorlevel = allmodules ("bundlecheck")
   end
   if errorlevel == 0 then
+    rmdir (ctandir)
     mkdir (ctandir .. "/" .. bundle)
+    rmdir (tdsdir)
     mkdir (tdsdir)
     if standalone then
       errorlevel = bundlectan ()
@@ -745,7 +821,9 @@ function ctan (standalone)
       end
     end
     dirzip (tdsdir, bundle .. ".tds")
-    cp (bundle .. ".tds.zip", tdsdir, ctandir)
+    if packtdszip then
+      cp (bundle .. ".tds.zip", tdsdir, ctandir)
+    end
     dirzip (ctandir, bundle)
     cp (bundle .. ".zip", ctandir, ".")
   else
@@ -791,6 +869,12 @@ function doc ()
     local name = stripext (file)
     -- A couple of short functions to deal with the repeated steps in a
     -- clear way
+    local function glossary (name)
+      run (
+          typesetdir ,
+          "makeindex -s gglo.ist -o " .. name .. ".gls " .. name .. ".glo"
+        )
+    end
     local function index (name)
       run (
           typesetdir ,
@@ -817,17 +901,20 @@ function doc ()
       print (" ! Compilation failed")
       return (errorlevel)
     else
-      for i = 1, 2 do -- Do twice
-        index (name)
-        typeset (file)
+      if fileexists (typesetdir .. "/" .. name .. ".idx") then
+       index (name)
       end
+      if fileexists (typesetdir .. "/" .. name .. ".glo") then
+        glossary (name)
+      end
+      typeset (file)
+      typeset (file)
       cp (name .. ".pdf", typesetdir, ".")
     end
     return (errorlevel)
   end
   -- Set up
-  cleandir (localdir)
-  mkdir (typesetdir)
+  cleandir (typesetdir)
   for _,i in ipairs (sourcefiles) do
     cp (i, ".", typesetdir)
   end
@@ -835,9 +922,10 @@ function doc ()
     cp (i, ".", typesetdir)
   end
   for _,i in ipairs (typesetsuppfiles) do
-    cp (i, supportdir, localdir)
+    cp (i, supportdir, typesetdir)
   end
   depinstall (typesetdeps)
+  unpack ()
   -- Main loop for doc creation
   for _,i in ipairs (typesetfiles) do
     for _,j in ipairs (filelist (".", i)) do
@@ -850,14 +938,6 @@ function doc ()
   return 0
 end
 
--- Unpack then install
-function localinstall ()
-  unpack ()
-  for _,i in ipairs (installfiles) do
-    cp (i, unpackdir, localdir)
-  end
-end
-
 -- Locally install files: only deals with those extracted, not docs etc.
 function install ()
   unpack ()
@@ -870,7 +950,7 @@ function install ()
   end
 end
 
-function savetlg (name, engine)
+function save (name, engine)
   local tlgfile = name .. (engine and ("." .. engine) or "") .. tlgext
   local newfile = name .. "." .. (engine or stdengine) .. logext
   if fileexists (testfiledir .. "/" .. name .. lvtext) then
@@ -892,13 +972,16 @@ end
 function unpack ()
   depinstall (unpackdeps)
   bundleunpack ()
+  for _,i in ipairs (installfiles) do
+    cp (i, unpackdir, localdir)
+  end
 end
 
 -- Split off from the main unpack so it can be used on a bundle and not
 -- leave only one modules files
 function bundleunpack ()
   mkdir (localdir)
-  mkdir (unpackdir)
+  cleandir (unpackdir)
   for _,i in ipairs (sourcefiles) do
     cp (i, ".", unpackdir)
   end
@@ -938,16 +1021,7 @@ function stdmain (target, file, engine)
   -- apart from ctan all of the targets are then just mappings
   if module == "" then
     -- Detect all of the modules
-    modules = { }
-    for entry in lfs.dir (".") do
-      if entry ~= "." and entry ~= ".." then
-        local attr = lfs.attributes (entry)
-        assert (type (attr) == "table")
-        if attr.mode == "directory" then
-          table.insert (modules, entry)
-        end
-      end
-    end
+    modules = modules or listmodules ()
     if target == "doc" then
       allmodules ("doc")
     elseif target == "check" then
@@ -965,8 +1039,6 @@ function stdmain (target, file, engine)
     elseif target == "install" then
       allmodules ("install")
     elseif target == "unpack" then
-      -- bundleunpack avoids cleaning out the dir, so do it once here
-      cleandir (unpackdir)
       allmodules ("bundleunpack")
     elseif target == "version" then
       version ()
@@ -985,13 +1057,7 @@ function stdmain (target, file, engine)
     elseif target == "doc" then
       doc ()
     elseif target == "check" and testfiledir ~= "" then
-      check ()
-    elseif target == "checklvt" and testfiledir ~= "" then
-      if file then
-        checklvt (file, engine)
-      else
-        help ()
-      end
+      check (file, engine)
     elseif target == "clean" then
       clean ()
     elseif target == "cmdcheck" and next (cmdchkfiles) ~= nil then
@@ -1000,11 +1066,9 @@ function stdmain (target, file, engine)
       ctan (true)
     elseif target == "install" then
       install ()
-    elseif target == "localinstall" then -- 'Hidden' target
-      localinstall ()
-    elseif target == "savetlg" and testfiledir ~= "" then
+    elseif target == "save" and testfiledir ~= "" then
       if file then
-        savetlg (file, engine)
+        save (file, engine)
       else
         help ()
       end
