@@ -4,8 +4,8 @@ $^W=1;
 
 my $prj     = 'latex-tds';
 my $file    = 'build.pl';
-my $version = '1.182';
-my $date    = '2014-02-03';
+my $version = '1.191';
+my $date    = '2014-05-21';
 my $author  = 'Heiko Oberdiek';
 my $copyright = "Copyright 2006-2014 $author";
 chomp(my $license = <<"END_LICENSE");
@@ -34,6 +34,7 @@ my $time_start = time;
 
 my $url_ctan = 'ftp://dante.ctan.org/tex-archive';
 my $url_ams = 'ftp://ftp.ams.org/pub/tex';
+my $url_amsbooka = 'http://tug.org/~karl/amsbooka.zip';
 my $url_ltxprj = 'http://www.latex-project.org/';
 
 my @required_list = (
@@ -66,15 +67,22 @@ my $error = "!!! Error:";
 my $dir_incoming = 'incoming';
 my $dir_incoming_ctan = "$dir_incoming/ctan";
 my $dir_incoming_ams = "$dir_incoming/ams";
+my $dir_incoming_amsbooka = "$dir_incoming/amsbooka";
 my $dir_incoming_ltxprj = "$dir_incoming/ltxprj";
+my $dir_ltxpub = "latex2e-public";
+my $dir_incoming_ltxpub = "$dir_incoming/$dir_ltxpub";
 my $dir_build = 'build';
 my $dir_lib = 'lib';
 my $dir_license = 'license';
 my $dir_tex = 'tex';
 my $dir_patch = 'patch';
 my $dir_distrib = 'distrib';
+my $dir_build_distrib = "$dir_build/distrib";
+my $dir_build_distrib_data = "$dir_build_distrib/$prj";
 my $dir_texmf = 'texmf';
-chomp(my $cwd = `pwd`);
+
+my $prg_pwd = 'pwd';
+chomp(my $cwd = `$prg_pwd`);
 
 # cache directory
 use File::HomeDir;
@@ -125,13 +133,14 @@ my $prg_lualatextds  = "lualatex -fmt=$cwd/$dir_build/lualatex-tds";
 my $prg_lualatextds2 = "lualatex -fmt=$cwd/$dir_build/lualatex-tds2";
 my $prg_pdflatex     = 'pdflatex';
 my $prg_pdflatextds  = "pdflatex -fmt=$cwd/$dir_build/pdflatex-tds";
-my $prg_pdftex       = "pdftex";
-my $prg_recode       = "recode";
-my $prg_rm           = "rm";
-my $prg_rsync        = "rsync";
+my $prg_pdftex       = 'pdftex';
+my $prg_recode       = 'recode';
+my $prg_rm           = 'rm';
+my $prg_rsync        = 'rsync';
 my $prg_sed          = "sed";
-my $prg_sort         = "sort";
-my $prg_texhash      = "texhash";
+my $prg_sort         = 'sort';
+my $prg_svn          = 'svn';
+my $prg_texhash      = 'texhash';
 my $prg_unzip        = 'unzip';
 my $prg_w3m          = 'w3m';
 my $prg_weave        = 'weave';
@@ -172,6 +181,8 @@ my $usage = <<"END_OF_USAGE";
 Usage: build.pl [options]
 General options:
   --(no)download      (check for newer files, disabled by default)
+  --(no)vcs           (use version control files)
+  --(no)vcs-update    (update version control files)
   --(no)postprocess   (pdf files are postprocessed, enabled by default)
   --(no)cache         (use cached pdf files, enabled by default)
 Module options:
@@ -180,6 +191,8 @@ END_OF_USAGE
 map { $usage .= "  --(no)$_\n"; } @pkg_list;
 
 my $opt_download    = 0;
+my $opt_vcs         = 0;
+my $opt_vcs_update  = 0;
 my $opt_postprocess = 0;
 my $opt_cache       = 1;
 my $opt_all         = 0;
@@ -195,6 +208,8 @@ GetOptions(
             map { $modules{$_} = 1; } @pkg_list;
         },
     'download!'    => \$opt_download,
+    'vcs!'         => \$opt_vcs,
+    'vcs-update!'  => \$opt_vcs_update,
     'postprocess!' => \$opt_postprocess,
     'cache!'       => \$opt_cache,
 ) or die $usage;
@@ -210,12 +225,32 @@ sub cd ($) {
     chdir $dir or die "$error Cannot change to directory `$dir'!\n";
 }
 
+### Line end sanitizing
+
+sub lf ($) {
+    my $file = shift;
+    open(IN, '<', $file) or die "!!! Error: Cannot open `$file'!\n";
+    my @lines = <IN>;
+    close(IN);
+    my $found = 0;
+    $found += s/\s+$/\n/ for @lines;
+    if ($found) {
+        open(OUT, '>', $file) or die "!!! Error: Cannot write `$file'!\n";
+        print OUT @lines;
+        close(OUT);
+        print "-->  lf($file): $found line(s) fixed.\n";
+    }
+    else {
+        print "--> lf($file): nothing to do.\n";
+    }
+}
+
 ### Format generation
 if (@list_modules > 0) {
     section('Format generation');
 
     ensure_directory($dir_build);
-    
+
     sub make_ini ($$) {
         my $prg = shift;
         my $fmt = shift;
@@ -229,12 +264,12 @@ if (@list_modules > 0) {
             run("$prg_cp $fmt.fmt $fmt.log $dir_cache");
         }
     }
-    
-    chdir $dir_build;
+
+    cd $dir_build;
     make_ini($prg_pdflatex, 'pdflatex-tds');
     make_ini($prg_lualatex, 'lualatex-tds');
     make_ini($prg_lualatex, 'lualatex-tds2');
-    chdir $cwd;
+    cd $cwd;
 }
 
 ### Download
@@ -318,8 +353,28 @@ if (@list_modules > 0) {
     download_err('lgc2');
     download_err('tlc2');
     download_ctan_file('armtex.zip', 'language/armenian');
+    ensure_directory($dir_incoming_amsbooka);
+    download("$dir_incoming_amsbooka/amsbooka.zip", $url_amsbooka);
 }
 
+### VCS
+{
+    section('VCS');
+
+    if ($opt_vcs_update) {
+        if (-d "$dir_incoming_ltxpub/.svn") {
+            cd $dir_incoming_ltxpub;
+            run("$prg_svn update");
+        }
+        else {
+            cd $dir_incoming;
+            run("$prg_svn checkout http://latex-project.org/svnroot/latex2e-public/ $dir_ltxpub");
+        }
+        cd $cwd;
+    }
+}
+
+
 ### Remove previous build
 section('Remove previous build');
 {
@@ -384,34 +439,66 @@ section('Unpacking');
         run("$prg_unzip -j $zipfile -d$dir");
     }
 
+    sub eols (@) {
+        for my $pat (@_) {
+            for my $file (glob "$dir_build/$pat") {
+                next if $file =~ /\.(?:pdf|zip)$/i;
+                lf $file;
+            }
+        }
+    }
+
     ensure_directory($dir_build);
-    unpack_ctan('base');
+
     if ($modules{'base'}) {
-        # replace .err files
-        foreach my $name (qw[
-            lb2
-            lgc2
-            manual
-            tlc2
-        ]) {
-            my $file = "$dir_incoming_ltxprj/$name.err";
-            my $dest = "$dir_build/base/$name.err";
-            run("$prg_cp $file $dest");
+        ensure_directory("$dir_build/base");
+        if ($opt_vcs) {
+            run("$prg_cp -r $dir_incoming_ltxpub/base/* $dir_build/base/");
+            run("$prg_cp $dir_incoming_ltxpub/doc/*.tex $dir_build/base/");
+        }
+        else {
+            unpack_ctan('base');
+            # replace .err files
+            foreach my $name (qw[
+                lb2
+                lgc2
+                manual
+                tlc2
+            ]) {
+                my $file = "$dir_incoming_ltxprj/$name.err";
+                my $dest = "$dir_build/base/$name.err";
+                run("$prg_cp $file $dest");
+            }
+            run("$prg_rm -rf $dir_build/base/doc");
+            unpacking('base',
+                      "$dir_incoming_ctan/doc.zip",
+                      "$dir_build/base");
+            run("$prg_cp -p $dir_build/base/doc/*.tex $dir_build/base/");
         }
-        run("$prg_rm -rf $dir_build/base/doc");
-        unpacking('base',
-                  "$dir_incoming_ctan/doc.zip",
-                  "$dir_build/base");
-        run("$prg_cp -p $dir_build/base/doc/*.tex $dir_build/base/");
         unpacking('base',
                   "$dir_incoming_ctan/armtex.zip",
                   "$dir_build/base");
+        eols("base/*") if $modules{'base'};
     }
-    map { 
-        unpack_ctan($_);
+
+    map {
+        ensure_directory("$dir_build/$_");
+        if ($opt_vcs and /cyrillic|graphics|tools/) {
+            run("$prg_cp -r $dir_incoming_ltxpub/required/$_/* $dir_build/$_/");
+        }
+        else {
+            unpack_ctan($_);
+        }
+        eols("$_/*") if $modules{$_};
     } @required_list;
+
     if ($modules{'amslatex'}) {
         unpack_ams('amscls', "$dir_incoming_ctan/amscls.tds.zip");
+        # Rename TDS:bibtex/bst/ams -> TDS:bibtex/bst/amscls
+        my $dir_bibtex_ams = "$dir_build/amslatex/texmf/bibtex/bst/ams";
+        my $dir_bibtex_amscls = "${dir_bibtex_ams}cls";
+        run("$prg_mv $dir_bibtex_ams $dir_bibtex_amscls") if
+                -d $dir_bibtex_ams and not -d $dir_bibtex_amscls;
         unpack_ams('amsrefs', "$dir_incoming_ctan/amsrefs.tds.zip");
         unpack_ams('amsmath', "$dir_incoming/ctan/math.tds.zip");
         #unpack_ams('amsrefs', "$dir_incoming_ams/amsrefs.zip");
@@ -426,7 +513,11 @@ section('Unpacking');
         ##        . "$dir_build/amslatex/texmf/source/latex/amsrefs/README");
         # run("$prg_cp $dir_build/amslatex/ctan/amsrefs/amsrefs.dtx "
         #         . "$dir_build/amslatex/texmf/source/latex/amsrefs/amsrefs.dtx");
+        unpacking_flat('amslatex',
+                       "$dir_incoming_amsbooka/amsbooka.zip",
+                       "$dir_build/amslatex");
     }
+
     if ($modules{'amsfonts'}) {
         ensure_directory("$dir_build/amsfonts");
         unpacking('amsfonts',
@@ -457,13 +548,13 @@ section('Patches');
     ; #
 
     if ($modules{'psnfss'}) {
-        chdir "$dir_build/psnfss";
+        cd "$dir_build/psnfss";
         run("$prg_checksum psfonts.dtx");
-        chdir $cwd;
+        cd $cwd;
     }
 
     if ($modules{'knuth'}) {
-        chdir "$dir_build/knuth";
+        cd "$dir_build/knuth";
         my @files = qw[
             trip.fot
             tripin.log
@@ -479,7 +570,7 @@ section('Patches');
             trap.typ
         ];
         run("$prg_chmod -x @files");
-        chdir $cwd;
+        cd $cwd;
     }
 
     if ($modules{'amslatex'}) {
@@ -496,18 +587,27 @@ section('Install source');
         my $pkg  = shift;
         my @list = @_;
         $modules{$pkg} or return 1;
-        chdir "$dir_build/$pkg";
+        cd "$dir_build/$pkg";
         install "texmf/source/$fmt/$pkg", @list;
-        chdir $cwd;
+        cd $cwd;
     }
     sub install_generic_source ($$@) {
         my $pkg = shift;
         my $dir = shift;
         my @list = @_;
         $modules{$pkg} or return 1;
-        chdir "$dir_build/$pkg";
+        cd "$dir_build/$pkg";
         install "texmf/source/$dir", @list;
-        chdir $cwd;
+        cd $cwd;
+    }
+    sub install_web_knuth ($@) {
+        my $pkg = 'knuth';
+        my $dir = shift;
+        my @list = @_;
+        $modules{$pkg} or return 1;
+        cd "$dir_build/$pkg";
+        install "texmf/web/$pkg/$dir", @list;
+        cd $cwd;
     }
     sub install_source ($@) {
         my $pkg = shift;
@@ -552,30 +652,34 @@ section('Install source');
         tds.sed
         tds.tex
     ]);
-    install_generic_source('knuth', 'knuth/texware', qw[
+    install_web_knuth('texware', qw[
         dvitype.web
         pltotf.web
         pooltype.web
         tftopl.web
     ]);
-    install_generic_source('knuth', 'knuth/mfware', qw[
+    install_web_knuth('mfware', qw[
         gftodvi.web
         gftype.web
         gftopk.web
         mft.web
     ]);
-    install_generic_source('knuth', 'knuth/etc', qw[
+    install_web_knuth('etc', qw[
         vptovf.web
         vftovp.web
     ]);
-    install_generic_source('knuth', 'knuth/web', qw[
+    install_web_knuth('web', qw[
         tangle.web
         weave.web
+    ]);
+    install_generic_source('knuth', 'knuth/web', qw[
         webman.tex
     ]);
-    install_generic_source('knuth', 'knuth/tex', qw[
+    install_web_knuth('tex', qw[
         glue.web
         tex.web
+    ]);
+    install_generic_source('knuth', 'knuth/tex', qw[
         trip.fot
         tripin.log
         trip.log
@@ -585,8 +689,10 @@ section('Install source');
         trip.tex
         trip.typ
     ]);
-    install_generic_source('knuth', 'knuth/mf', qw[
+    install_web_knuth('mf', qw[
         mf.web
+    ]);
+    install_generic_source('knuth', 'knuth/mf', qw[
         trap.fot
         trapin.log
         trap.log
@@ -611,13 +717,21 @@ section('Install source');
         errorlog.tex
         logmac.tex
     ]);
+    if ($modules{'amslatex'}) {
+        cd "$dir_build/amslatex";
+        install "texmf/source/latex/amscls", qw[
+            amsbooka.dtx
+            amsbooka.ins
+        ];
+        cd $cwd;
+    }
 }
 
 ### Patch source files after source install
 section('Patches after source install');
 {
     if ($modules{'base'}) {
-        chdir "$dir_build/base";
+        cd "$dir_build/base";
 
         # ltdirchk.dtx must be patched to fool it in
         # not having texsys.cfg
@@ -635,31 +749,14 @@ section('Patches after source install');
             close(IN);
         }
 
-        # base: TDS:makeindex/base -> TDS:makeindex/latex
-        {
-            my $file_ins = 'docstrip.ins';
-            my $file_org = 'docstrip.ins.org';
-            rename $file_ins, $file_org;
-            open(IN, '<', $file_org) or die "$error Cannot open `$file_org'!\n";
-            open(OUT, '>', $file_ins) or die "$error Cannot write `$file_ins'!\n";
-            while (<IN>) {
-                s|makeindex/base|makeindex/latex|;
-                print OUT;
-            }
-            close(OUT);
-            close(IN);
-        }
-
-        # fix CR/LF line endings of tlc2.err (2012-05-12)
-        run("$prg_zip -ll tlc2.zip tlc2.err");
-        run("$prg_unzip -o tlc2.zip");
-
-        chdir $cwd;
+        cd $cwd;
 
+        lf("$dir_build/base/encguide.tex");
         patch('base/encguide.tex');
         patch('base/source2e.tex');
         patch('base/tlc2.err');
         patch('base/utf8ienc.dtx');
+        #
         # lb2.err contains <CR><LF> line endings, a patch file
         # created by diff in Linux would create mixed line endings
         # causing trouble for subversion (Karl Berry).
@@ -671,7 +768,6 @@ section('Patches after source install');
     }
 
     if ($modules{'knuth'}) {
-
         foreach my $file (qw[
             errorlog.tex
             logmac.tex
@@ -681,7 +777,6 @@ section('Patches after source install');
         ]) {
             patch("knuth/$file");
         }
-
     }
 
     if ($modules{'amslatex'}) {
@@ -703,9 +798,9 @@ section('Docstrip');
         my $pkg = shift;
         my $ins = shift;
         $modules{$pkg} or return 1;
-        chdir "$dir_build/$pkg";
+        cd "$dir_build/$pkg";
         run("$prg_docstrip $ins.ins");
-        chdir $cwd;
+        cd $cwd;
         1;
     }
     docstrip('base',     'unpack');
@@ -714,12 +809,13 @@ section('Docstrip');
     docstrip('graphics', 'graphics');
     docstrip('graphics', 'graphics-drivers');
     docstrip('tools',    'tools');
+    docstrip('amslatex', 'amsbooka');
 
     ## patch for amsthm.sty, part 1/2
     #if ($modules{'amslatex'}) {
-    #    chdir "$dir_build/amslatex/amscls";
+    #    cd "$dir_build/amslatex/amscls";
     #    run("$prg_docstrip ams-c1.ins");
-    #    chdir $cwd;
+    #    cd $cwd;
     #}
 }
 
@@ -757,17 +853,28 @@ section('TDS cleanup');
           #   cite-x*.tex jr.bib
           map { run("$prg_mv $_ $dir_tds/doc/latex/amsrefs/"); } @list;
         }
-        # CTAN:macros/latex/required/amslatex/other/*
-        run("$prg_cp $dir_build/amslatex/other/amsbooka.sty"
-            . " $dir_build/amslatex/texmf/tex/latex/amscls/amsbooka.sty");
     }
 }
 
 ### Install TDS/tex, TDS/doc files
 section('Install tex doc');
 {
+    sub check_readme ($) {
+        my $module = shift;
+        # my $readme = $module eq 'tools' ? 'readme.txt' : '00readme.txt';
+        my $readme = '00readme.txt'; # since LaTeX revision 307
+        if (-f "README" and not -f $readme) {
+            run("$prg_cp README $readme");
+        }
+        -f $readme or
+                die "!!! Errror($module): Missing readme!\n";
+        -f "readme.txt" and -f "00readme.txt" and
+                die "!!! Error($module): Duplicate readme!\n";
+    }
+
     if ($modules{'base'}) {
-        chdir "$dir_build/base";
+        cd "$dir_build/base";
+        check_readme 'base';
         install 'texmf/doc/latex/base', qw[
             00readme.txt
             autoload.txt
@@ -793,40 +900,43 @@ section('Install tex doc');
         install 'texmf/tex/latex/base', qw[
             texsys.cfg
         ];
-        chdir $cwd;
+        cd $cwd;
     }
 
     if ($modules{'tools'}) {
-        chdir "$dir_build/tools";
+        cd "$dir_build/tools";
+        check_readme 'tools';
         install 'texmf/doc/latex/tools', qw[
             changes.txt
             manifest.txt
-            readme.txt
+            00readme.txt
         ];
-        chdir $cwd;
+        cd $cwd;
     }
 
     if ($modules{'graphics'}) {
-        chdir "$dir_build/graphics";
+        cd "$dir_build/graphics";
+        check_readme 'graphics';
         install('texmf/doc/latex/graphics',
             '*.txt'
         );
         install('texmf/tex/latex/graphics',
             '*.def'
         );
-        chdir $cwd;
+        cd $cwd;
     }
 
     if ($modules{'cyrillic'}) {
-        chdir "$dir_build/cyrillic";
+        cd "$dir_build/cyrillic";
+        check_readme 'cyrillic';
         install('texmf/doc/latex/cyrillic',
             '*.txt'
         );
-        chdir $cwd;
+        cd $cwd;
     }
 
     if ($modules{'psnfss'}) {
-        chdir "$dir_build/psnfss";
+        cd "$dir_build/psnfss";
         install('texmf/doc/latex/psnfss',
             '*.txt'
         );
@@ -839,11 +949,11 @@ section('Install tex doc');
         install('texmf/fonts/map/dvips/psnfss',
             '*.map'
         );
-        chdir $cwd;
+        cd $cwd;
     }
 
     if ($modules{'tds'}) {
-        chdir "$dir_build/tds";
+        cd "$dir_build/tds";
         install('texmf/doc/tds', qw[
             README
             ChangeLog
@@ -852,11 +962,11 @@ section('Install tex doc');
         install('texmf/doc/info', qw[
             tds.info
         ]);
-        chdir $cwd;
+        cd $cwd;
     }
 
     if ($modules{'knuth'}) {
-        chdir "$dir_build/knuth";
+        cd "$dir_build/knuth";
         install('texmf/doc/knuth/tex', qw[
             texbook.tex
         ]);
@@ -868,11 +978,11 @@ section('Install tex doc');
             mf84.bug
             tex82.bug
         ]);
-        chdir $cwd;
+        cd $cwd;
     }
 
     if ($modules{'etex'}) {
-        chdir "$dir_build/etex";
+        cd "$dir_build/etex";
         my $doc_dir = 'texmf/doc/etex/base';
         my $src_dir = 'texmf/source/etex/base';
         ensure_directory($doc_dir);
@@ -880,31 +990,29 @@ section('Install tex doc');
         install($src_dir, qw[
             etex_man.tex etex_man.sty
         ]);
-        chdir $cwd;
+        cd $cwd;
+    }
+
+    if ($modules{'amslatex'}) {
+        cd "$dir_build/amslatex";
+        install('texmf/tex/latex/amscls', 'amsbooka.sty');
+        cd $cwd;
     }
 
     ## patch for amsthm.sty, part 2/2
     #if ($modules{'amslatex'}) {
-    #    chdir "$dir_build/amslatex/amscls";
+    #    cd "$dir_build/amslatex/amscls";
     #    my $dest_dir = '../texmf/tex/latex/amscls';
     #    ensure_directory($dest_dir);
     #    install($dest_dir, 'amsthm.sty');
-    #    chdir $cwd;
+    #    cd $cwd;
     #}
 }
 
 ### Preparation for documentation
 if ($modules{'base'}) {
     my $dir_src = "$dir_build/base/armtex";
-    my $tds_mf = "$dir_texmf/fonts/source/public/armtex";
-    my $tds_tfm = "$dir_texmf/fonts/tfm/public/armtex";
-    my $tds_latex = "$dir_texmf/tex/latex/armtex";
-    my @tds_mf = map {chomp;$_} glob "$dir_src/mf/*.mf";
-    my @tds_tfm = map {chomp;$_} glob "$dir_src/tfm/*.tfm";
-    my @tds_latex = map {chomp;$_} glob "$dir_src/latex/*.*";
-    install($tds_mf, @tds_mf);
-    install($tds_tfm, @tds_tfm);
-    install($tds_latex, @tds_latex);
+    run("$prg_cp -r $dir_build/base/armtex/* $dir_texmf/");
     run("$prg_texhash $dir_texmf");
 }
 
@@ -951,6 +1059,7 @@ if ($modules{'base'}) {
         cache 'base', $guide, sub {
             my $latextds = $prg_lualatextds;
             $latextds = $prg_lualatextds2 if $guide eq 'usrguide';
+            $latextds = $prg_pdflatextds if $guide eq 'encguide';
             run("$latextds -draftmode $guide");
             run("$latextds -draftmode $guide");
             final_begin;
@@ -1011,7 +1120,7 @@ if ($modules{'base'}) {
         install_pdf('base', "$base");
         1;
     }
-    chdir "$dir_build/base";
+    cd "$dir_build/base";
 
     ## source2e
     cache 'base', 'source2e', sub {
@@ -1165,19 +1274,20 @@ END_CODE
         }
     }
 
-    chdir $cwd;
+    cd $cwd;
 }
 
 ### Generate documentation for tools
 if ($modules{'tools'}) {
     section('Documentation: tools');
 
-    chdir "$dir_build/tools";
+    cd "$dir_build/tools";
     my @list = glob("*.dtx");
     map { s/\.dtx$//; } @list;
     foreach my $entry (@list) {
         my $latextds = $prg_lualatextds;
-        $latextds = $prg_pdflatextds if $entry eq 'bm';
+        $latextds = $prg_pdflatextds if $entry eq 'bm'
+                                     or $entry eq 'multicol';
         $latextds = $prg_lualatextds2 if $entry eq 'calc'
                                       or $entry eq 'rawfonts'
                                       or $entry eq 'showkeys';
@@ -1279,14 +1389,14 @@ END_FILE
         final_end;
     };
     install_pdf('tools', 'tools');
-    chdir $cwd;
+    cd $cwd;
 }
 
 ### Generate documentation for cyrillic
 if ($modules{'cyrillic'}) {
     section('Documentation: cyrillic');
 
-    chdir "$dir_build/cyrillic";
+    cd "$dir_build/cyrillic";
     my @list = (glob("*.dtx"), glob("*.fdd"));
     foreach my $entry (@list) {
         my $base = $entry;
@@ -1300,14 +1410,14 @@ if ($modules{'cyrillic'}) {
         };
         install_pdf('cyrillic', $base);
     }
-    chdir $cwd;
+    cd $cwd;
 }
 
 ### Generate documentation for graphics
 if ($modules{'graphics'}) {
     section('Documentation: graphics');
 
-    chdir "$dir_build/graphics";
+    cd "$dir_build/graphics";
     my @list = glob("*.dtx");
     map { s/\.dtx$//; } @list;
     foreach my $entry (@list) {
@@ -1336,7 +1446,7 @@ END_CODE
         final_end;
     };
     install_pdf('graphics', 'grfguide');
-    chdir $cwd;
+    cd $cwd;
 }
 
 sub makeindex ($) {
@@ -1394,23 +1504,27 @@ if ($modules{'amslatex'}) {
         install_pdf($amspkg, $doc);
     }
 
-    chdir "$dir_build/amslatex/amsmath";
+    cd "$dir_build/amslatex/amsmath";
     symlink '../texmf', 'texmf';
     map { generate_doc 'amsmath', $_; } qw[
         amsldoc subeqn technote testmath
         amsbsy amscd amsgen amsmath amsopn amstext amsxtra
     ];
-    chdir $cwd;
+    cd $cwd;
 
-    chdir "$dir_build/amslatex/amscls";
+    cd "$dir_build/amslatex/amscls";
     symlink '../texmf', 'texmf';
     map { generate_doc 'amscls', $_; } qw[
         amsthdoc instr-l thmtest
         amsclass amsdtx amsmidx upref
     ];
-    chdir $cwd;
+    cd $cwd;
 
-    chdir "$dir_build/amslatex/amsrefs";
+    cd "$dir_build/amslatex";
+    generate_doc 'amscls', 'amsbooka';
+    cd $cwd;
+
+    cd "$dir_build/amslatex/amsrefs";
     symlink '../texmf', 'texmf';
     map { generate_doc 'amsrefs', $_; } qw[
         amsrdoc changes
@@ -1418,7 +1532,7 @@ if ($modules{'amslatex'}) {
     ];
     # 2013-02-13: Excluded as test files:
     #   cite-xa cite-xb cite-xh cite-xs
-    chdir $cwd;
+    cd $cwd;
 }
 
 if ($modules{'amsfonts'}) {
@@ -1449,7 +1563,7 @@ if ($modules{'amsfonts'}) {
         install_gen_pdf('fonts', $amspkg, $doc);
     }
 
-    chdir "$dir_build/amsfonts";
+    cd "$dir_build/amsfonts";
     map {generate_doc2 'amsfonts', $_; } qw[
         amsfonts amssymb cmmib57 eufrak euscript
     ];
@@ -1464,14 +1578,14 @@ if ($modules{'amsfonts'}) {
         };
     }
     install_gen_pdf('fonts', 'amsfonts', 'amsfndoc');
-    chdir $cwd;
+    cd $cwd;
 }
 
 ### Generate documentation for psnfss
 if ($modules{'psnfss'}) {
     section('Documentation: psnfss');
 
-    chdir "$dir_build/psnfss";
+    cd "$dir_build/psnfss";
 
     cache 'psnfss', 'psfonts', sub {
         run("$prg_pdflatextds -draftmode psfonts.dtx");
@@ -1490,14 +1604,14 @@ if ($modules{'psnfss'}) {
     };
     install_pdf('psnfss', 'psnfss2e');
 
-    chdir $cwd;
+    cd $cwd;
 }
 
 ### Generate documentation for tds
 if ($modules{'tds'}) {
     section('Documentation: tds');
 
-    chdir "$dir_build/tds";
+    cd "$dir_build/tds";
 
     my $file_tds = 'tds.tex';
     my $file_tds_new = 'tds.new';
@@ -1530,14 +1644,14 @@ END_TEXT
     };
     install_gen_pdf('', 'tds', 'tds');
 
-    chdir $cwd;
+    cd $cwd;
 }
 
 ### Generate documentation for knuth
 if ($modules{'knuth'}) {
     section('Documentation: knuth');
 
-    chdir "$dir_build/knuth";
+    cd "$dir_build/knuth";
 
     my $knuth_drv = "$cwd/$dir_tex/knuth.drv";
 
@@ -1676,14 +1790,14 @@ if ($modules{'knuth'}) {
     };
     install_gen_pdf('knuth', 'errata', 'errata');
 
-    chdir $cwd;
+    cd $cwd;
 }
 
 ### Generate documentation for etex
 if ($modules{'etex'}) {
     section('Documentation: etex');
 
-    chdir "$dir_build/etex";
+    cd "$dir_build/etex";
 
     my $entry = 'etex_man';
     my $etex_man_drv = "$cwd/$dir_tex/$entry.drv";
@@ -1697,7 +1811,7 @@ if ($modules{'etex'}) {
     };
     install_gen_pdf('etex', 'base', $entry);
 
-    chdir $cwd;
+    cd $cwd;
 }
 
 ### Module source
@@ -1718,6 +1832,7 @@ if ($modules{'source'}) {
 
     # generate README.html
     run("$prg_asciidoc --backend=xhtml11 README.asciidoc");
+    lf("README.html");
 
     # generate README.pdf
     # run("$prg_asciidoc --out-file=$file_readme_notoc_html"
@@ -1760,7 +1875,7 @@ if ($modules{'source'}) {
             push @lines_post, $_;
             push @lines_post, ($2 ? '-' : '=') x length($_) . "\n";
             my $prefix = ' ' x (3 * ($2 ? 2 : 1));
-            s/^(\d+)\.(\d+\.)? /($1<10 ? ' ' : '') . "$1.$2 "/e;
+            s/^(\d+)\.((?:\d+\.)?) /($1<10 ? ' ' : '') . "$1.$2 "/e;
             push @lines_toc, "$prefix$_";
             next;
         }
@@ -1846,13 +1961,29 @@ section('Distrib');
     }
 
     if ($opt_all) {
-        chdir $dir_distrib;
-        my $cmd = "$prg_zip -0 $file_ctan_distrib README README.html README.pdf";
-        for my $pkg (sort @pkg_list) {
-            $cmd .= " $pkg.tds.zip";
+        ensure_directory($dir_build_distrib);
+        my @files = map {"$cwd/$_"} qw[
+                README
+                README.html
+                README.pdf
+            ],
+            map {"$dir_distrib/$_.tds.zip"} sort @pkg_list;
+        run("$prg_rm -rf $dir_build_distrib_data") if -d $dir_build_distrib_data;
+        ensure_directory($dir_build_distrib_data);
+        cd $dir_build_distrib_data;
+        my @list;
+        foreach my $file (@files) {
+            $file =~ m|/([^/]+)$| or die "!!! Error: Cannot extract file name (`$file')!\n";
+            my $file_new = $1;
+            -f $file or die "!!! Error: File `$file' not found!\n";
+            link $file, $file_new or die "!!! Error: Cannot link `$file' => `$file_new': $!\n";
+            push @list, "$prj/$file_new";
         }
+        cd $cwd;
+        cd $dir_build_distrib;
+        my $cmd = "$prg_zip -0 $file_ctan_distrib $prj @list";
         run($cmd);
-        chdir $cwd;
+        cd $cwd;
     }
 }
 
